<?php


/**
 * Instances of this class are injected as $api argument into hook_pageapi implementations.
 */
class pageapi_InjectedAPI {

  protected $_vars;
  protected $_module;
  protected $_module_path;

  function __construct(&$vars) {
    $this->_vars =& $vars;
  }

  function favicon($file, $location = TRUE) {
    $file = $this->_buildFilepath($file, $location);
    $this->_vars['favicon'] = base_path() . $file;
  }

  /**
   * Shortcut for drupal_add_css()
   *
   * @param $file :string
   *   Filepath, relative to the module dir.
   * @param $module :string
   *   Location specifier. Can be
   *   - TRUE, to specify the current module.
   *   - "module:[module name]", to specify a module.
   *   - "theme:[theme name]", to specify a theme.
   *   - "library:[library name]", to specify a library (http://drupal.org/project/libraries).
   *   - "files", if $file is relative to the files dir.
   *   - FALSE, if $file is relative to drupal root.
   */
  function css($file, $location = TRUE) {
    $file = $this->_buildFilepath($file, $location);
    drupal_add_css($file);
  }

  /**
   * Shortcut for drupal_add_js()
   *
   * @param $file :string
   *   Filepath, relative to the module dir.
   * @param $location :string
   *   Location specifier. Same syntax as in the css() method.
   */
  function js($file, $location = TRUE) {
    $file = $this->_buildFilepath($file, $location);
    drupal_add_js($file);
  }

  /**
   * Determine the physical filepath, relative to Drupal root.
   *
   * @param $file :string
   *   Filepath, relative to the module dir.
   * @param $location :string
   *   Location specifier, same syntax as in the css() method.
   * @return :string
   *   Filepath, relative to Drupal root.
   */
  protected function _buildFilepath($file, $location) {
    if ($location === TRUE) {
      $dir = $this->_module_path;
    }
    elseif (!is_string($location)) {
      $dir = NULL;
    }
    else {
      list($type, $name) = explode(':', $location) + array(NULL);
      if (!isset($name)) {
        if ($type == 'files') {
          $dir = file_directory_path();
          $download_method = variable_get('file_downloads', FILE_DOWNLOADS_PUBLIC);
          if ($download_method == FILE_DOWNLOADS_PRIVATE) {
            // TODO: Support private download method.
            // This is not trivial, because now we can no longer use the simple way of url building.
            // The $dir variable should remain the same, anyway, because this is only about the physical location.
          }
        }
        elseif ($type == 'site') {
          // Current site dir.
          $dir = conf_path();
        }
      }
      else {
        if ($type == 'module') {
          $dir = drupal_get_path('module', $name);
          if (empty($dir)) {
            throw new Exception("Module '$name' does not exist, or is not enabled.");
          }
        }
        elseif ($type == 'theme') {
          $dir = drupal_get_path('theme', $name);
          if (empty($dir)) {
            throw new Exception("Theme '$name' does not exist, or is not enabled.");
          }
        }
        elseif ($type == 'library') {
          $dir = libraries_get_path($name);
          if (empty($dir)) {
            throw new Exception("Library '$name' was not found.");
          }
        }
        else {
          throw new Exception("Unknown location type '$type' in '$location'.");
        }
      }
      if (empty($dir)) {
        throw new Exception("Invalid location specifier '$location'.");
      }
    }
    return isset($dir) ? ($dir . '/' . $file) : $file;
  }

  function jsSetting($key, $value) {
    $pieces = array_reverse(explode('.', $key));
    while ($piece = array_pop($pieces)) {
      $value = array($piece => $value);
    }
    drupal_add_js($value, array('type' => 'setting'));
  }

  function jsSettings(array $values) {
    drupal_add_js($values, array('type' => 'setting'));
  }

  /**
   * @param $path :string
   *   The system path of the targeted local task (tab).
   * @param $weight :numeric or FALSE
   *   FALSE, to disable the local task.
   *   numeric, to give it a new weight.
   */
  function moveLocalTask($route, $weight) {
    $this->_vars['localTaskCache']->moveLocalTask($route, $weight);
  }

  /**
   * This function is used behind the scenes, to allow pageapi to know about the
   * module that owns the currently running hook_pageapi implementation.
   *
   * Only use this, if you want to pass around the $api object to other modules.
   */
  function setModule($module) {
    $this->_module = $module;
    $this->_module_path = drupal_get_path('module', $module);
  }

  /**
   * This function is used behind the scenes, to allow pageapi to know about the
   * theme that owns the currently running hook_pageapi implementation.
   *
   * Only use this, if you want to pass around the $api object to other themes.
   */
  function setTheme($theme) {
    $this->_module = $theme;
    $this->_module_path = drupal_get_path('theme', $theme);
  }
}
